## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup_AbS, warning = FALSE, message = FALSE, eval = FALSE, results="hide"----
# # Load required package
# library(AbSolution)
# 
# # Load Shiny app
# AbSolution::run_app()
# 

