---
title: "Report v.1.00"
author: "AbSolution"
date: "`r format(Sys.time())`"
output: 
  html_document:
    toc: true # table of content true
    toc_depth: 3  # upto three depths of headings (specified by #, ## and ###)
    toc_float: true
    number_sections: true  ## if you want number sections at each table header
    theme: flatly  # many options for theme, this one is my favorite.
    highlight: tango  # specifies the syntax highlighting style
    code_folding: hide
---




```{r setup, include = FALSE}
knitr::opts_chunk$set(
  out.width = "100%", 
  tidy = TRUE
)
{{setupRmarkdown2}}
```


**AbSolution** is a tool for BCR/TCR Fab sequence feature analysis. This report was generated for reproducibility purposes.<br>
<br>
For more details on using AbSolution see <http://rmarkdown.rstudio.com>. AbSolution follows the [ENCORE guidelines](https://www.nature.com/articles/s41467-024-52446-8). <br>

Author: `r (author)`<br>
Contact email: `r (usermail)`

# Project information

<b>User comments:</b> `r (user_0_comments)`<br>

```{r loading_info, warning = FALSE, message = FALSE}
{{loading}}

```

<b>Datasets and parameters used:</b>
```{r loading_parameters, warning = FALSE, message = FALSE, max.height='200px'}
{{parameter_setup}}
```

# Processing input 

<br><b>User comments:</b> `r (user_1_comments)`<br> `r (user_2_comments)`

```{r parse_input, warning = FALSE, message = FALSE, eval = FALSE}
{{parse_input}}
```

# Feature calculation

<br><b>User comments:</b> `r (user_2_comments)`<br> 
```{r feature_calculation, warning = FALSE, message = FALSE, eval = FALSE}
{{feature_calculation}}
```

```{r loading_file, warning = FALSE, message = FALSE}
{{file_load}}
```


# Clonal exploration

```{r clonal_explor, warning = FALSE, message = FALSE}
{{clone}}
```
<br><b>User comments:</b> `r (user_5_comments)`

# Sequence exploration

```{r sequence_selection, warning = FALSE, message = FALSE}
{{PCA_ex}}
```
<br><b>User comments:</b> `r (user_3_comments)`

# Feature exploration

```{r feature, warning = FALSE, message = FALSE}
{{feature}}
```
<br><b>User comments:</b> `r (user_4_comments)`

# Citation
## AbSolution
## ENCORE
> *van Kampen AHC, Mahamune U, Jongejan A, van Schaik BDC, Balashova D, Lashgari D, Pras-Raves M, Wever EJM, Dane AD, García-Valiente R, Moerland PD. ENCORE: a practical implementation to improve reproducibility and transparency of computational research. Nat Commun. 2024 Sep 16;15(1):8117. doi: 10.1038/s41467-024-52446-8. PMID: 39284801; PMCID: PMC11405857.*

# System and version information

## System information
```{r system_info, warning = FALSE, message = FALSE}
          osVersion
          Sys.info()[which(names(Sys.info()) %in% c("sysname", "release", "version", "machine"))]
          print(benchmarkme::get_ram(), unit_system = "iec")
          benchmarkme::get_cpu()
```

## R information
```{r r_info, warning = FALSE, message = FALSE}
          sessionInfo()

```


<br><br><br><br>

![](data:image/png;base64,[encoded data, 27704 characters])





<link rel="icon" type="image/png" sizes="16x16" href="data:image/png;base64,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" />
<link rel="shortcut icon" type="image/png" sizes="16x16" href="data:image/png;base64,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" />
